@@ -24,6 +24,7 @@ import (
 	"k8s.io/client-go/tools/pager"
 	watchutil "k8s.io/client-go/tools/watch"
 	"k8s.io/klog/v2/klogr"
+	"k8s.io/kubectl/pkg/util/openapi"
 
 	"github.com/argoproj/gitops-engine/pkg/utils/kube"
 	"github.com/argoproj/gitops-engine/pkg/utils/tracing"
@@ -65,6 +66,8 @@ type ClusterInfo struct {
 	LastCacheSyncTime *time.Time
 	// SyncError holds most recent cache synchronization error
 	SyncError error
+	// APIGroups holds list of API groups supported by the cluster
+	APIGroups []metav1.APIGroup
 }
 
 // OnEventHandler is a function that handles Kubernetes event
@@ -84,6 +87,8 @@ type ClusterCache interface {
 	GetServerVersion() string
 	// GetAPIGroups returns information about observed API groups
 	GetAPIGroups() []metav1.APIGroup
+	// GetOpenAPISchema returns open API schema of supported API resources
+	GetOpenAPISchema() openapi.Resources
 	// Invalidate cache and executes callback that optionally might update cache settings
 	Invalidate(opts ...UpdateSettingsFunc)
 	// FindResources returns resources that matches given list of predicates from specified namespace or everywhere if specified namespace is empty
@@ -160,17 +165,19 @@ type clusterCache struct {
 	resources map[kube.ResourceKey]*Resource
 	nsIndex   map[string]map[kube.ResourceKey]*Resource
 
-	kubectl    kube.Kubectl
-	log        logr.Logger
-	config     *rest.Config
-	namespaces []string
-	settings   Settings
+	kubectl          kube.Kubectl
+	log              logr.Logger
+	config           *rest.Config
+	namespaces       []string
+	clusterResources bool
+	settings         Settings
 
 	handlersLock                sync.Mutex
 	handlerKey                  uint64
 	populateResourceInfoHandler OnPopulateResourceInfoHandler
 	resourceUpdatedHandlers     map[uint64]OnResourceUpdatedHandler
 	eventHandlers               map[uint64]OnEventHandler
+	openAPISchema               openapi.Resources
 }
 
 type clusterCacheSync struct {
@@ -239,10 +246,18 @@ func (c *clusterCache) GetServerVersion() string {
 
 // GetAPIGroups returns information about observed API groups
 func (c *clusterCache) GetAPIGroups() []metav1.APIGroup {
+	c.lock.RLock()
+	defer c.lock.RUnlock()
+
 	return c.apiGroups
 }
 
-func (c *clusterCache) AppendAPIGroups(apiGroup metav1.APIGroup) {
+// GetOpenAPISchema returns open API schema of supported API resources
+func (c *clusterCache) GetOpenAPISchema() openapi.Resources {
+	return c.openAPISchema
+}
+
+func (c *clusterCache) appendAPIGroups(apiGroup metav1.APIGroup) {
 	exists := false
 	for i := range c.apiGroups {
 		if c.apiGroups[i].Name == apiGroup.Name {
@@ -557,7 +572,7 @@ func (c *clusterCache) watchEvents(ctx context.Context, api kube.APIResourceInfo
 					} else {
 						// add new CRD's groupkind to c.apigroups
 						if event.Type == watch.Added && nameOK && nameErr == nil {
-							c.AppendAPIGroups(apiGroup)
+							c.appendAPIGroups(apiGroup)
 						}
 						err = runSynced(&c.lock, func() error {
 							return c.startMissingWatches()
@@ -566,6 +581,17 @@ func (c *clusterCache) watchEvents(ctx context.Context, api kube.APIResourceInfo
 							c.log.Error(err, "Failed to start missing watch")
 						}
 					}
+					err = runSynced(&c.lock, func() error {
+						openAPISchema, err := c.kubectl.LoadOpenAPISchema(c.config)
+						if err != nil {
+							return err
+						}
+						c.openAPISchema = openAPISchema
+						return nil
+					})
+					if err != nil {
+						c.log.Error(err, "Failed to reload open api schema")
+					}
 				}
 			}
 		}
@@ -574,20 +600,20 @@ func (c *clusterCache) watchEvents(ctx context.Context, api kube.APIResourceInfo
 
 func (c *clusterCache) processApi(client dynamic.Interface, api kube.APIResourceInfo, callback func(resClient dynamic.ResourceInterface, ns string) error) error {
 	resClient := client.Resource(api.GroupVersionResource)
-	if len(c.namespaces) == 0 {
+	switch {
+	// if manage whole cluster or resource is cluster level and cluster resources enabled
+	case len(c.namespaces) == 0 || !api.Meta.Namespaced && c.clusterResources:
 		return callback(resClient, "")
-	}
-
-	if !api.Meta.Namespaced {
-		return nil
-	}
-
-	for _, ns := range c.namespaces {
-		err := callback(resClient.Namespace(ns), ns)
-		if err != nil {
-			return err
+	// if manage some namespaces and resource is namespaced
+	case len(c.namespaces) != 0 && api.Meta.Namespaced:
+		for _, ns := range c.namespaces {
+			err := callback(resClient.Namespace(ns), ns)
+			if err != nil {
+				return err
+			}
 		}
 	}
+
 	return nil
 }
 
@@ -612,6 +638,13 @@ func (c *clusterCache) sync() error {
 		return err
 	}
 	c.apiGroups = groups
+
+	openAPISchema, err := c.kubectl.LoadOpenAPISchema(config)
+	if err != nil {
+		return err
+	}
+	c.openAPISchema = openAPISchema
+
 	apis, err := c.kubectl.GetAPIResources(c.config, c.settings.ResourcesFilter)
 
 	if err != nil {
@@ -769,12 +802,6 @@ func (c *clusterCache) IsNamespaced(gk schema.GroupKind) (bool, error) {
 }
 
 func (c *clusterCache) managesNamespace(namespace string) bool {
-	if len(c.namespaces) == 0 {
-		return true
-	}
-	if namespace == "" {
-		return false
-	}
 	for _, ns := range c.namespaces {
 		if ns == namespace {
 			return true
@@ -791,11 +818,12 @@ func (c *clusterCache) GetManagedLiveObjs(targetObjs []*unstructured.Unstructure
 	defer c.lock.RUnlock()
 
 	for _, o := range targetObjs {
-		if !c.managesNamespace(o.GetNamespace()) {
-			if o.GetNamespace() == "" {
+		if len(c.namespaces) > 0 {
+			if o.GetNamespace() == "" && !c.clusterResources {
 				return nil, fmt.Errorf("Cluster level %s %q can not be managed when in namespaced mode", o.GetKind(), o.GetName())
+			} else if o.GetNamespace() != "" && !c.managesNamespace(o.GetNamespace()) {
+				return nil, fmt.Errorf("Namespace %q for %s %q is not managed", o.GetNamespace(), o.GetKind(), o.GetName())
 			}
-			return nil, fmt.Errorf("Namespace %q for %s %q is not managed", o.GetNamespace(), o.GetKind(), o.GetName())
 		}
 	}
 
@@ -942,6 +970,7 @@ func (c *clusterCache) GetClusterInfo() ClusterInfo {
 		Server:            c.config.Host,
 		LastCacheSyncTime: c.syncStatus.syncTime,
 		SyncError:         c.syncStatus.syncError,
+		APIGroups:         c.apiGroups,
 	}
 }
 
